/*
 *  SokoBan, a Sokoban game.
 *  Copyright (C) 2008  Colomban Wendling <ban@herbesfolles.org>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/* This file content game-sepecific functions */


#include <stdlib.h>

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include "sokoban.h"
#include "map.h"
#include "player.h"
#include "level.h"
#include "dialog.h"
#include "game.h"
#include "i18n.h"
#include "set.h"


/**
 * \defgroup game Game specific functions
 * Functions used only by the game part
 */


/** @{ */


/**
 * \brief Check if the player have won
 * \return TRUE if the player have won, FALSE else
 */
MyBool game_getWon (void)
{
   int i, j;
   
   for (i = 0; i < map_getHeight (); i++)
   {
      for (j = 0; j < map_getWidth (); j++)
      {
         if (map_checkBlockAtPos (j, i, POINT))
            return FALSE;
      }
   }
   
   return TRUE;
}


static void game_render (SDL_Surface *screen)
{
   map_blit (screen);
   map_blitObject (OBJECT (player_get ()), screen);
   
   SDL_Flip (screen);
}


void game_main (SDL_Surface *screen)
{
   int play = 1;
   int n = 0;
   SDL_Event ev;
   
   player_init ();
   
   /* try to load set */
   if (set_open (DATADIR"index.set") != 0)
   {
      player_free ();
      return;
   }
   
   /* try to load first level of the set */
   if (set_loadLevel (n) != 0)
   {
      set_close ();
      player_free ();
      return;
   }
   else
      game_render (screen); /* init level screen */
   
   
   while (SDL_WaitEvent (&ev) && play)
   {
      switch (ev.type)
      {
         case SDL_QUIT:
            play = 0;
            continue;
            break;
         
         case SDL_KEYDOWN:
            switch (ev.key.keysym.sym)
            {
               /* exit level (exit game loop, back to menu) */
               case SDLK_q:
               case SDLK_ESCAPE:
                  play = 0;
                  continue;
                  break;
               
               /* reload level */
               case SDLK_r:
                  set_loadLevel (n);
                  break;
               
               /* go to next level */
               case SDLK_n:
               case SDLK_PAGEDOWN:
                  if (set_loadLevel (++n) != 0)
                     set_loadLevel (--n); /* reload previous level */
                  break;
               
               /* go to previous level */
               case SDLK_p:
               case SDLK_PAGEUP:
                  if (set_loadLevel (--n) != 0)
                     set_loadLevel (++n); /* reload previous level */
                  break;
               
               /* move player */
               case SDLK_i:
               case SDLK_KP8:
               case SDLK_UP:
                  player_move (UP);
                  break;
               
               case SDLK_k:
               case SDLK_KP2:
               case SDLK_DOWN:
                  player_move (DOWN);
                  break;
               
               case SDLK_j:
               case SDLK_KP4:
               case SDLK_LEFT:
                  player_move (LEFT);
                  break;
               
               case SDLK_l:
               case SDLK_KP6:
               case SDLK_RIGHT:
                  player_move (RIGHT);
                  break;
               
               /* nothing */
               default:
                  continue; /* don't render if no changes */
                  break;
            }
            break;
         
         default:
            continue; /* don't render if no changes */
            break;
      }
      
      game_render (screen);
      
      if (game_getWon ())
      {
         int loop = 1;
         
         #ifdef DEBUG
         printf ("you won!\n");
         #endif
         
         printf ("tux moves: %d; box moves: %d (score: %d)\n",
                 player_getMoves (), player_getBoxMoves (),
                 player_getMoves () + (player_getBoxMoves () * 2));
         
         /* display dialog that show to the player that hi won */
         dialog_setFontSize (BIG);
         dialog_info (screen, screen->w, screen->h, _("You won!"));
         SDL_Flip (screen);
         
         /* wait for user action */
         while (SDL_WaitEvent (&ev) && loop)
         {
            switch (ev.type)
            {
               case SDL_QUIT:
                  loop = 0;
                  continue;
                  break;
               
               case SDL_KEYDOWN:
                  switch (ev.key.keysym.sym)
                  {
                     case SDLK_r:
                        n --; /* reload same level (see below) */
                     
                     /* exit won loop */
                     case SDLK_p:      /* play */
                     case SDLK_q:      /* quit */
                     case SDLK_RETURN: /* OK */
                     case SDLK_ESCAPE: /* quit */
                        loop = 0;
                        continue;
                        break;
                     
                     default:
                        continue;
                  }
               
               default:
                  continue;
            }
         }
         
         /* load next level or quit */
         if (set_loadLevel (++n) != 0)
            break;
         else
            game_render (screen);
      }
   }
   
   set_close ();
   
   player_free ();
}


/** @} */
